#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define ISOLAT2_CHARS_COUNT	30
#define CP1250_CHARS_COUNT	6

#define UTF8_CHARS_COUNT	ISOLAT2_CHARS_COUNT
#define UTF8_PREFIXES_COUNT	3

const unsigned char isolat2[ISOLAT2_CHARS_COUNT] = "ةݮ";
const unsigned char isolat2_target[ISOLAT2_CHARS_COUNT] = "ACDEEINORSTUUYZacdeeinorstuuyz";

const unsigned char cp1250[CP1250_CHARS_COUNT] = "";
const unsigned char cp1250_target[CP1250_CHARS_COUNT] = "STZstz";

const char *utf8[UTF8_CHARS_COUNT] = {
    "Á",
    "Č",
    "Ď",
    "É",
    "Ě",
    "Í",
    "Ň",
    "Ó",
    "Ř",
    "Š",
    "Ť",
    "Ú",
    "Ů",
    "Ý",
    "Ž",
    "á",
    "č",
    "ď",
    "é",
    "ě",
    "í",
    "ň",
    "ó",
    "ř",
    "š",
    "ť",
    "ú",
    "ů",
    "ý",
    "ž"
};
const unsigned char utf8_prefixes[UTF8_PREFIXES_COUNT] = "";
const unsigned char utf8_target[UTF8_CHARS_COUNT] = "ACDEEINORSTUUYZacdeeinorstuuyz";

typedef struct _tree_type {
	int is_item;
	void *next[256];
} tree_type;

typedef struct _item_type {
	int is_item;
	char ascii;
} item_type;

typedef union _node_type {
	int is_item;
	tree_type tree;
	item_type item;
} node_type;

node_type root_node;

void
compile()
{
	int counter;
	node_type *new_node;

	memset((void *)&root_node, 0, sizeof(tree_type));
	for (counter = 0; counter < ISOLAT2_CHARS_COUNT; counter++) {
		if (root_node.tree.next[isolat2[counter]]) {
			fprintf(stderr, "compile: conflict in char '%c' %x!\n", isolat2[counter], (unsigned int)isolat2[counter]);
		} else {
			new_node = (node_type *)malloc(sizeof(item_type));
			new_node->is_item = 1;
			new_node->item.ascii = isolat2_target[counter];
			root_node.tree.next[isolat2[counter]] = new_node;
		}
	}
	for (counter = 0; counter < CP1250_CHARS_COUNT; counter++) {
		if (root_node.tree.next[cp1250[counter]]) {
			fprintf(stderr, "compile: conflict in char '%c' %x!\n", cp1250[counter], (unsigned int)cp1250[counter]);
		} else {
			new_node = (node_type *)malloc(sizeof(item_type));
			new_node->is_item = 1;
			new_node->item.ascii = cp1250_target[counter];
			root_node.tree.next[cp1250[counter]] = new_node;
		}
	}
	for (counter = 0; counter < UTF8_CHARS_COUNT; counter++) {
		if (root_node.tree.next[(unsigned char)utf8[counter][0]]) {
			if (((node_type *)root_node.tree.next[(unsigned char)utf8[counter][0]])->is_item) {
				fprintf(stderr, "compile: conflict in char '%c' %x!\n", (unsigned char)utf8[counter][0], (unsigned int)utf8[counter][0]);
			} else {
				new_node = (node_type *)malloc(sizeof(item_type));
				new_node->is_item = 1;
				new_node->item.ascii = utf8_target[counter];
				if (((node_type *)root_node.tree.next[(unsigned char)utf8[counter][0]])->tree.next[(unsigned char)utf8[counter][1]]) {
					fprintf(stderr, "compile: conflict in char '%c%c' %x %x!\n", (unsigned char)utf8[counter][0], (unsigned char)utf8[counter][1], (unsigned int)utf8[counter][0], (unsigned int)utf8[counter][1]);
				} else {
					((node_type *)root_node.tree.next[(unsigned char)utf8[counter][0]])->tree.next[(unsigned char)utf8[counter][1]] = new_node;
				}
			}
		} else {
			new_node = (node_type *)malloc(sizeof(tree_type));
			memset((void *)new_node, 0, sizeof(tree_type));
			root_node.tree.next[(unsigned char)utf8[counter][0]] = new_node;
			new_node = (node_type *)malloc(sizeof(item_type));
			new_node->is_item = 1;
			new_node->item.ascii = utf8_target[counter];
			((node_type *)root_node.tree.next[(unsigned char)utf8[counter][0]])->tree.next[(unsigned char)utf8[counter][1]] = new_node;
		}
	}
}

void
clean_up(node_type *node)
{
	int counter;

	if (node->is_item)
		return;
	for (counter = 0; counter < 256; counter++)
		if (node->tree.next[counter]) {
			clean_up((node_type *)node->tree.next[counter]);
			free(node->tree.next[counter]);
		}
}

int
main()
{
	int result, next;
	node_type *current;

	compile();

	while (!feof(stdin)) {
		result = getc(stdin);
		if (result == EOF)
			break;
		else {
			if (result > 127) {
				if ((current = (node_type *)root_node.tree.next[(unsigned char)result])) {
					if (current->is_item)
						putc(current->item.ascii, stdout);
					else {
						next = getc(stdin);
						if (next == EOF)
							break;
						if ((current = (node_type *)current->tree.next[(unsigned char)next]) &&
						current->is_item)
							putc(current->item.ascii, stdout);
					}
				}
				/* Invalid chars are ignored */
			} else
				putc(result, stdout);
		}
	}

	clean_up(&root_node);
	return 0;
}
